#!/usr/bin/python3
import re
import os
import dbus
import json
import psutil
import signal

STRATEGIES_DBUS_PATH = '/com/kylin/UpgradeStrategies'
STRATEGIES_DBUS_SERVICE = 'com.kylin.UpgradeStrategies'
STRATEGIES_DBUS_INTERFACE = 'com.kylin.UpgradeStrategies.interface'

process_names = [
    "kylin-upgrade-strategies",
    "kconf2 monitor kylin-update-frontend."]

def _quit_strategies_service():
    try:
        bus = dbus.SystemBus()
        the_other_guy = bus.get_object(STRATEGIES_DBUS_SERVICE,
                                        STRATEGIES_DBUS_PATH)
        the_other_guy.Quit(dbus_interface=STRATEGIES_DBUS_INTERFACE,
                            timeout=2)
    except Exception as e:
        pass

def kill_process_tree(pid, sig=signal.SIGTERM):
    try:
        parent = psutil.Process(pid)
    except psutil.NoSuchProcess:
        return

    children = parent.children(recursive=True)
    for child in children:
        try:
            child.send_signal(sig)
        except psutil.NoSuchProcess:
            pass

    try:
        parent.send_signal(sig)
    except psutil.NoSuchProcess:
        pass

def _quit_force_strategies_service():
    try:
        for proc in psutil.process_iter(['pid', 'name', 'cmdline']):
            cmdline_str = ' '.join(proc.info['cmdline'])

            if any(name in cmdline_str for name in process_names):
                print(f"正在终止进程 {cmdline_str} - {proc}")
                kill_process_tree(proc.info['pid'])
    except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
        pass

def _restore_limit_policy():
    RATE_CONFIG_FILE    = "/var/cache/kylin-system-updater/rate_limites.json"
    limit_file          = "/etc/apt/apt.conf.d/80apt-download"
    limit_value         = None

    try:
        pattern = re.compile(r'Acquire::(http|https)::Dl-Limit\s+"(\d+)";')

        if os.path.exists(limit_file):
            with open(limit_file, 'r') as f:
                content = f.read()

                matches = pattern.findall(content)
                for protocol, limit in matches:
                    if protocol == 'http':
                        limit_value = int(limit)
                        break

            if limit_value:
                rate_config = {
                    "policy_limits": False,
                    "rate_limits": [],
                    "default_download": str(limit_value)
                }

                with open(RATE_CONFIG_FILE, 'w') as config_file:
                    json.dump(rate_config, config_file, indent=4)

            os.remove("/etc/apt/apt.conf.d/80apt-download")

    except Exception as e:
        pass

if __name__ == "__main__":
    _quit_strategies_service()
    _quit_force_strategies_service()
    _restore_limit_policy()